<?php

namespace Drupal\metatags_quick\Plugin\field\field_type;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'metatags_quick' field type.
 *
 * @FieldType(
 *   id = "metatags_quick",
 *   label = @Translation("Meta tag"),
 *   description = @Translation("This field stores meta tags."),
 *   instance_settings = {
 *     "min" = "",
 *     "max" = "",
 *     "prefix" = "",
 *     "suffix" = "",
 *     "meta_name" = "",
 *   },
 *   default_widget = "metatags_quick_default",
 *   default_formatter = "metatag_formatter_default"
 * )
 */
class MetatagItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Value'))
      ->setRequired(FALSE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'length' => 1024,
          'not null' => FALSE,
        ],
      ],
    ];
  }

}
